@@ -13,7 +13,7 @@
 //!
 //! This crate offers optional dependencies on the following crates to enable [`Protocol`] implementations for some of their types:
 //!
-//! * [`chrono`](https://docs.rs/chrono): [`DateTime`](https://docs.rs/chrono/latest/chrono/struct.DateTime.html), [`Utc`](https://docs.rs/chrono/latest/chrono/offset/struct.Utc.html), and [`FixedOffset`](https://docs.rs/chrono/latest/chrono/offset/struct.FixedOffset.html)
+//! * [`chrono`](https://docs.rs/chrono): [`NaiveDate`](https://docs.rs/chrono/latest/chrono/naive/struct.NaiveDate.html), [`DateTime`](https://docs.rs/chrono/latest/chrono/struct.DateTime.html), [`Utc`](https://docs.rs/chrono/latest/chrono/offset/struct.Utc.html), and [`FixedOffset`](https://docs.rs/chrono/latest/chrono/offset/struct.FixedOffset.html)
 //! * [`chrono-tz`](https://docs.rs/chrono-tz): [`Tz`](https://docs.rs/chrono-tz/latest/chrono_tz/enum.Tz.html)
 //! * [`noisy_float`](https://docs.rs/noisy_float): [`NoisyFloat`](https://docs.rs/noisy_float/latest/noisy_float/struct.NoisyFloat.html)
 //! * [`serde_json`](https://docs.rs/serde_json): [`Value`](https://docs.rs/serde_json/latest/serde_json/enum.Value.html), [`Map`](https://docs.rs/serde_json/latest/serde_json/struct.Map.html), and [`Number`](https://docs.rs/serde_json/latest/serde_json/struct.Number.html)
@@ -28,6 +28,7 @@
 
 use {
     std::{
+        convert::Infallible,
         fmt,
         future::Future,
         io,
@@ -98,6 +99,12 @@ pub enum ReadError {
     Warp(Arc<dep_warp::Error>),
 }
 
+impl From<Infallible> for ReadError {
+    fn from(never: Infallible) -> Self {
+        match never {}
+    }
+}
+
 impl From<io::Error> for ReadError {
     fn from(e: io::Error) -> ReadError {
         ReadError::Io(Arc::new(e))
@@ -163,6 +170,12 @@ pub enum WriteError {
     Warp(Arc<dep_warp::Error>),
 }
 
+impl From<Infallible> for WriteError {
+    fn from(never: Infallible) -> Self {
+        match never {}
+    }
+}
+
 impl From<io::Error> for WriteError {
     fn from(e: io::Error) -> WriteError {
         WriteError::Io(Arc::new(e))
@@ -204,8 +217,16 @@ impl std::error::Error for WriteError {} //TODO use thiserror for better sources
 /// This trait allows reading a value of an implementing type from an async or sync stream, as well as writing one to an async or sync sink.
 pub trait Protocol: Sized {
     /// Reads a value of this type from an async stream.
+    ///
+    /// # Cancellation safety
+    ///
+    /// Implementations of this method are generally not cancellation safe.
     fn read<'a, R: AsyncRead + Unpin + Send + 'a>(stream: &'a mut R) -> Pin<Box<dyn Future<Output = Result<Self, ReadError>> + Send + 'a>>;
     /// Writes a value of this type to an async sink.
+    ///
+    /// # Cancellation safety
+    ///
+    /// Implementations of this method are generally not cancellation safe.
     fn write<'a, W: AsyncWrite + Unpin + Send + 'a>(&'a self, sink: &'a mut W) -> Pin<Box<dyn Future<Output = Result<(), WriteError>> + Send + 'a>>;
     #[cfg(feature = "read-sync")]
     #[cfg_attr(docsrs, doc(cfg(feature = "read-sync")))]
@@ -282,16 +303,25 @@ pub trait Protocol: Sized {
     #[cfg(feature = "tokio-tungstenite")]
     #[cfg_attr(docsrs, doc(cfg(feature = "tokio-tungstenite")))]
     /// Reads a value of this type from a [`tokio-tungstenite`](dep_tokio_tungstenite) websocket.
+    ///
+    /// # Cancellation safety
+    ///
+    /// The default implementation of this method is cancellation safe if and only if the `read-sync` feature is enabled.
     fn read_ws<'a, R: Stream<Item = Result<dep_tokio_tungstenite::tungstenite::Message, dep_tokio_tungstenite::tungstenite::Error>> + Unpin + Send + 'a>(stream: &'a mut R) -> Pin<Box<dyn Future<Output = Result<Self, ReadError>> + Send + 'a>> {
         Box::pin(async move {
             let packet = stream.try_next().await?.ok_or(ReadError::EndOfStream)?;
-            Self::read(&mut &*packet.into_data()).await
+            #[cfg(feature = "read-sync")] { Self::read_sync(&mut &*packet.into_data()) }
+            #[cfg(not(feature = "read-sync"))] { Self::read(&mut &*packet.into_data()).await }
         })
     }
 
     #[cfg(feature = "tokio-tungstenite")]
     #[cfg_attr(docsrs, doc(cfg(feature = "tokio-tungstenite")))]
     /// Writes a value of this type to a [`tokio-tungstenite`](dep_tokio_tungstenite) websocket.
+    ///
+    /// # Cancellation safety
+    ///
+    /// The default implementation of this method is not cancellation safe.
     fn write_ws<'a, W: Sink<dep_tokio_tungstenite::tungstenite::Message, Error = dep_tokio_tungstenite::tungstenite::Error> + Unpin + Send + 'a>(&'a self, sink: &'a mut W) -> Pin<Box<dyn Future<Output = Result<(), WriteError>> + Send + 'a>>
     where Self: Sync {
         Box::pin(async move {
@@ -305,16 +335,25 @@ pub trait Protocol: Sized {
     #[cfg(feature = "warp")]
     #[cfg_attr(docsrs, doc(cfg(feature = "warp")))]
     /// Reads a value of this type from a [`warp`](dep_warp) websocket.
+    ///
+    /// # Cancellation safety
+    ///
+    /// The default implementation of this method is cancellation safe if and only if the `read-sync` feature is enabled.
     fn read_warp<'a, R: Stream<Item = Result<dep_warp::filters::ws::Message, dep_warp::Error>> + Unpin + Send + 'a>(stream: &'a mut R) -> Pin<Box<dyn Future<Output = Result<Self, ReadError>> + Send + 'a>> {
         Box::pin(async move {
             let packet = stream.try_next().await?.ok_or(ReadError::EndOfStream)?;
-            Self::read(&mut packet.as_bytes()).await
+            #[cfg(feature = "read-sync")] { Self::read_sync(&mut packet.as_bytes()) }
+            #[cfg(not(feature = "read-sync"))] { Self::read(&mut packet.as_bytes()).await }
         })
     }
 
     #[cfg(feature = "warp")]
     #[cfg_attr(docsrs, doc(cfg(feature = "warp")))]
     /// Writes a value of this type to a [`warp`](dep_warp) websocket.
+    ///
+    /// # Cancellation safety
+    ///
+    /// The default implementation of this method is not cancellation safe.
     fn write_warp<'a, W: Sink<dep_warp::filters::ws::Message, Error = dep_warp::Error> + Unpin + Send + 'a>(&'a self, sink: &'a mut W) -> Pin<Box<dyn Future<Output = Result<(), WriteError>> + Send + 'a>>
     where Self: Sync {
         Box::pin(async move {
